#include "testlib.h"
#include <map>
#include <set>

#define verdict(x) (x == -1 ? _fail : _wa)

std::map<std::string, int> nouns, verbs, adjs;

void proceed_sentence(std::string const &sentence, int &words, int &score, std::set<std::string> &occured_words, int const test) {
    int pos = 0, length = (int)sentence.length();
    while (pos != length && sentence[pos] == ' ') {
        ++pos;
    }
    if (pos == length) {
        quitf(_wa, "%d%s test. Empty sentence.\n", test + 1, englishEnding(test + 1).c_str());
    }
    
    if (!isupper(sentence[pos])) {
        quitf(_wa, "%d%s test. First letter must be a capital.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
    }
    if (!islower(sentence[length - 1])) {
        quitf(_wa, "%d%s test. Last symbol must be a letter.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
    }
    
    std::string word(1, tolower(sentence[pos]));
    bool is_word = true, noun_occured = false, last_adjective = false, verb_occured = false;
    
    for (++pos; pos <= length; ++pos) {
        if (pos != length && islower(sentence[pos])) {
            if (is_word) {
                word += sentence[pos];
            } else {
                word = sentence[pos];
                is_word = true;
            }
        } else if (pos == length || sentence[pos] == ' ') {
            if (is_word) {
                if (occured_words.find(word) != occured_words.end()) {
                    quitf(_wa, "%d%s test. Word '%s' was used at least twice.\n", test + 1, englishEnding(test + 1).c_str(), word.c_str());
                }
                occured_words.insert(word);
                
                if (nouns.find(word) != nouns.end()) {
                    noun_occured = true;
                    last_adjective = false;
                    score += nouns[word];
                } else if (verbs.find(word) != verbs.end()) {
                    if (verb_occured) {
                        if (occured_words.find(word) != occured_words.end()) {
                            quitf(_wa, "%d%s test. More than one verb is the sentence.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
                        }
                    }
                    if (last_adjective) {
                        quitf(_wa, "%d%s test. An adjective is followed by a verb.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
                    }
                    verb_occured = true;
                    score += verbs[word];
                } else if (adjs.find(word) != adjs.end()) {
                    if (last_adjective) {
                        quitf(_wa, "%d%s test. An adjective is followed by an adjective.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
                    }
                    last_adjective = true;
                    score += adjs[word];
                } else {
                    quitf(_wa, "%d%s test. Word '%s' is unknown.", test + 1, englishEnding(test + 1).c_str(), word.c_str());
                }
                
                ++words;
                is_word = false;
            }
        } else {
            quitf(_wa, "%d%s test. Bad symbol '%c' at position %d.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence[pos], pos + 1, sentence.c_str());
        }
    }
    
    if (last_adjective) {
        quitf(_wa, "%d%s test. An adjective is the last word.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
    }
    if (!noun_occured) {
        quitf(_wa, "%d%s test. There is no noun in the sentence.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
    }
    if (!verb_occured) {
        quitf(_wa, "%d%s test. There is no verb in the sentence.\n%s.", test + 1, englishEnding(test + 1).c_str(), sentence.c_str());
    }
}

int get_answer(InStream &stream, int left, int right, int test) {
    std::string line = stream.readLine();
    std::set<std::string> occured_words;
    
    if (line.empty()) {
        quitf(_wa, "%d%s test. The text is empty.\n", test + 1, englishEnding(test + 1).c_str());
    }
    
    if (!isalpha(line[0])) {
        quitf(_wa, "%d%s test. First symbol must be an alpha.\n", test + 1, englishEnding(test + 1).c_str());
    }
    
    int end = (int)line.length();
    while (line[end - 1] == ' ') {
        --end;
    }
    if (line[end - 1] != '.') {
        quitf(_wa, "%d%s test. Last symbol must be a full stop.\n", test + 1, englishEnding(test + 1).c_str());
    }
    
    int pos = 0, words = 0, score = 0;
    
    while (pos != end) {
        int full_stop = (int)line.find('.', pos);
        proceed_sentence(line.substr(pos, full_stop - pos), words, score, occured_words, test);
        pos = full_stop + 1;
        
        if (pos != end && line[pos] != ' ') {
            quitf(_wa, "%d%s test. Symbol %d after a sentence must be a space, but '%c' found.\n", test + 1, englishEnding(test + 1).c_str(), pos + 1, line[pos]);
        }
    }
    
    if (words < left || words > right) {
        quitf(_wa, "%d%s test. Number of words is not in bounds: found %d, expected [%d, %d].\n",
              test + 1, englishEnding(test + 1).c_str(), words, left, right);
    }
    
    return score;
}

int main(int argc, char **argv) {
    setName("moscow 2015 essay checker");
    registerTestlibCmd(argc, argv);
    
    int tests = inf.readInt();
    
    for (int test = 0; test != tests; ++test) {
        int n = inf.readInt(), v = inf.readInt(), a = inf.readInt();
        for (int i = 0; i != n; ++i) {
            nouns[inf.readWord()] = inf.readInt();
        }
        for (int i = 0; i != v; ++i) {
            verbs[inf.readWord()] = inf.readInt();
        }
        for (int i = 0; i != a; ++i) {
            adjs[inf.readWord()] = inf.readInt();
        }
        
        int left = inf.readInt(), right = inf.readInt();
        
        int ja = get_answer(ans, left, right, -1);
        int pa = get_answer(ouf, left, right, test);
        
        if (pa > ja) {
            quitf(_fail, "%d%s test. Participant got %d points, despite judge answer is %d.", test + 1, englishEnding(test + 1).c_str(),  pa, ja);
        }
        if (pa < ja) {
            quitf(_wa, "%d%s test. Participant got %d points, although %d is possible.", test + 1, englishEnding(test + 1).c_str(), pa, ja);
        }
        
        nouns.clear();
        verbs.clear();
        adjs.clear();
    }
    
    quitp(30.0, "%d tests are passed.", tests);
}
